/*
 * tut10.c -- SDL-Tutorial, 10. Kapitel
 * NO Copyright (c) by Nicolai 'Prefect' Haehnle
 *
 * Diese Software ist Public Domain.
 */

#include <stdlib.h>
#include <stdio.h>

#include <SDL.h>
#include <SDL_opengl.h>

unsigned g_texPicture;
unsigned g_texShip;

int gl_state_texenable = 0;
unsigned gl_state_texture = 0;

/* Aktiviert oder deaktiviert das Texturieren */
static __inline__ void GL_TexEnable(int enable)
{
	if (gl_state_texenable == enable)
		return;

	if (enable)
		glEnable(GL_TEXTURE_2D);
	else
		glDisable(GL_TEXTURE_2D);

	gl_state_texenable = enable;
}

/* Bindet eine Textur */
static __inline__ void GL_Bind(unsigned texture)
{
	if (gl_state_texture == texture)
		return;

	glBindTexture(GL_TEXTURE_2D, texture);

	gl_state_texture = texture;
}


/* Ldt eine Textur aus einer .bmp-Datei */
unsigned LoadTexture(const char *filename, int withcolorkey, int r, int g, int b)
{
	SDL_Surface *bitmap, *conv;
	unsigned texture;
	int format;

/* Lade die Datei */
	bitmap = SDL_LoadBMP(filename);
	if (!bitmap) {
		fprintf(stderr, "%s: konnte nicht geladen werden\n", filename);
		exit(1);
	}

	if (withcolorkey) {
		Uint32 color = SDL_MapRGB(bitmap->format, (Uint8)r, (Uint8)g, (Uint8)b);
		SDL_SetColorKey(bitmap, SDL_SRCCOLORKEY, color);
	}

/* Konvertiere ins 32-Bit RGBA-Format */
	conv = SDL_CreateRGBSurface(SDL_SWSURFACE, bitmap->w, bitmap->h, 32,
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
			0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
#else
			0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
#endif
	SDL_BlitSurface(bitmap, 0, conv, 0);

/* Erstelle eine OpenGL-Textur */
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glPixelStorei(GL_UNPACK_ROW_LENGTH, conv->pitch / conv->format->BytesPerPixel);

	if (withcolorkey)
		format = 4;
	else
		format = 3;
	glTexImage2D(GL_TEXTURE_2D, 0, format, conv->w, conv->h, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, conv->pixels);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);

/* Gib die verwendeten Surfaces und Speicherplatz frei */
	SDL_FreeSurface(bitmap);
	SDL_FreeSurface(conv);

	return texture;
}

/* Blittet eine Textur auf den Bildschirm */
void Blit(unsigned texture, int x, int y, int w, int h)
{
	GL_TexEnable(1);
	GL_Bind(texture);

	glBegin(GL_QUADS);
		glTexCoord2f(0, 0);	glVertex2i(x, y);
		glTexCoord2f(1, 0);	glVertex2i(x+w, y);
		glTexCoord2f(1, 1);	glVertex2i(x+w, y+h);
		glTexCoord2f(0, 1);	glVertex2i(x, y+h);
	glEnd();
}

void RefreshScreen()
{
	glColor3ub(255, 255, 255);

	glDisable(GL_ALPHA_TEST);

	Blit(g_texPicture, 0, 0, 640, 480);

	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0);

	Blit(g_texShip, 320, 170, 64, 64);
}

int main(int argc, char **argv)
{
	SDL_Surface *screen;
	int running;

/* SDL initialisieren */
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "SDL konnte nicht initialisiert werden: %s\n",
			SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

/* OpenGL-Bildschirmmodus */
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	screen = SDL_SetVideoMode(640, 480, 0, SDL_OPENGL);
	if (!screen) {
		fprintf(stderr, "Konnte Bildschirmmodus nicht setzen: %s\n",
			SDL_GetError());
		exit(1);
	}

/* Lade Texturen */
	g_texPicture = LoadTexture("galaxien.bmp", 0, 0, 0, 0);
	g_texShip = LoadTexture("ship.bmp", 1, 255, 0, 255);

/* Initialisierung von OpenGL */
	glViewport(0, 0, screen->w, screen->h);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, screen->w, screen->h, 0, -1.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

/* Programmschleife */
	running = 1;
	while(running) {
		SDL_Event event;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				running = 0;
				break;
			case SDL_QUIT:
				running = 0;
				break;
			}
		}


		RefreshScreen();

		// Pageflip: Back- und Frontbuffer werden vertauscht
		SDL_GL_SwapBuffers();
	}

/* Gib die OpenGL-Texturen frei */
	glDeleteTextures(1, &g_texPicture);
	glDeleteTextures(1, &g_texShip);

        return 0;
}
